@@ -362,7 +362,7 @@ int HttpRequest::_on_body(http_parser* pParser, const char* pAt, size_t length)
 
   // Copy pAt because the source data is deleted right after calling this
   // function.
-  boost::shared_ptr<std::vector<char>> buf = boost::make_shared<std::vector<char>>(pAt, pAt + length);
+  boost::shared_ptr<std::vector<char> > buf = boost::make_shared<std::vector<char> >(pAt, pAt + length);
 
   boost::function<void(boost::shared_ptr<HttpResponse>)> schedule_bg_callback(
     boost::bind(&HttpRequest::_schedule_on_body_error, shared_from_this(), _1)
@@ -414,11 +414,20 @@ void HttpRequest::_on_body_error(boost::shared_ptr<HttpResponse> pResponse) {
 // Message complete
 // ============================================================================
 
+bool HttpRequest::isUpgrade() const {
+  // Normally this should just be _parser.upgrade. But we also want to allow
+  // Upgrade: WebSocket + Connection: close, in order to work around an issue
+  // in RStudio Server's http proxying code with Firefox (only):
+  // https://github.com/rstudio/rstudio/issues/2940
+  // https://github.com/rstudio/shiny/issues/2064
+  return _parser.upgrade || _parser.flags & F_UPGRADE;
+}
+
 int HttpRequest::_on_message_complete(http_parser* pParser) {
   ASSERT_BACKGROUND_THREAD()
   trace("HttpRequest::_on_message_complete");
 
-  if (pParser->upgrade)
+  if (isUpgrade())
     return 0;
 
   boost::function<void(boost::shared_ptr<HttpResponse>)> schedule_bg_callback(
@@ -490,7 +499,7 @@ void HttpRequest::onWSMessage(bool binary, const char* data, size_t len) {
 
   // Copy data because the source data is deleted right after calling this
   // function.
-  boost::shared_ptr<std::vector<char>> buf = boost::make_shared<std::vector<char>>(data, data + len);
+  boost::shared_ptr<std::vector<char> > buf = boost::make_shared<std::vector<char> >(data, data + len);
 
   boost::function<void (void)> error_callback(
     boost::bind(&HttpRequest::schedule_close, shared_from_this())
@@ -675,7 +684,7 @@ void HttpRequest::_call_r_on_ws_open() {
   // _requestBuffer is likely empty at this point, but copy its contents and
   // _pass along just in case.
 
-  boost::shared_ptr<std::vector<char>> req_buffer = boost::make_shared<std::vector<char>>(_requestBuffer);
+  boost::shared_ptr<std::vector<char> > req_buffer = boost::make_shared<std::vector<char> >(_requestBuffer);
   _requestBuffer.clear();
 
 
@@ -706,7 +715,7 @@ void HttpRequest::_parse_http_data(char* buffer, const ssize_t n) {
     // buffer.
     _requestBuffer.insert(_requestBuffer.end(), buffer + parsed, buffer + n);
 
-  } else if (_parser.upgrade) {
+  } else if (isUpgrade()) {
     char* pData = buffer + parsed;
     size_t pDataLen = n - parsed;
 
